/**
 * \file i2c.c
 *
 * \brief PIC24F IC driver source file
 *
 * Copyright (c) 2016 Measurement Specialties. All rights reserved.
 *
 */

#include "i2c.h"

#define TIMEOUT	500

static uint16_t timeout;

void i2c_idle(void) {
	timeout = TIMEOUT;
	while((_SEN || _RSEN || _PEN || _RCEN || _ACKEN || _R_W) && --timeout);  
}

void i2c_transmit(void) {
	timeout = TIMEOUT;
	while(_TBF && _TRSTAT && --timeout);
}

void i2c_receive(void) {
	timeout = TIMEOUT;
    while(_RCEN && --timeout);
    _RCEN = 0;
}

void i2c_ack(void) {
	timeout = TIMEOUT;
    while( _ACKEN  && --timeout);	
}

void i2c_start(void) {
    _SEN = 1;	
    i2c_idle();
}

void i2c_stop(void) {
    i2c_idle();
    _PEN = 1; 
    while(_PEN && --timeout);
}

bool i2c_write_byte(uint8_t txByte) {
    I2C1TRN = txByte;
    i2c_transmit();
	i2c_idle();
    return _ACKSTAT == 0;	
}

uint8_t i2c_read_byte(bool nack) {
    _RCEN = 1;
    i2c_receive();
    _I2COV = 0;
    _ACKDT = nack == true ? 1 : 0;
    _ACKEN = 1;
    i2c_ack();
    uint8_t byte = I2C1RCV;
    return byte;
}

void i2c_master_init(void) {
    TRISGbits.TRISG2 = 1;
    TRISGbits.TRISG3 = 1;
    
    I2C1CON = 0x0000; // disable I2C
    I2C1STAT = 0x0000;
    I2C1TRN = 0x0000;
    I2C1RCV = 0x0000;

    I2C1BRG = I2C_BRG;
    I2C1CON = 0x8200;
	
	i2c_idle();
}

enum status_code i2c_master_read_packet_wait(struct i2c_master_packet *const packet) {
	
	i2c_start();
	if (i2c_write_byte((packet->address << 1) | I2C_TRANSFER_READ) == true) {
		for (int i = 0; i < packet->data_length; i++) {
			packet->data[i] = i2c_read_byte((i + 1) == packet->data_length);
		}
	}
	else {
		i2c_stop();
		return STATUS_ERR_OVERFLOW;
	}
	i2c_stop();
	return STATUS_OK;
}

enum status_code i2c_master_write_packet_wait_no_stop(struct i2c_master_packet *const packet) {
	i2c_start();
	if (i2c_write_byte((packet->address << 1) | I2C_TRANSFER_WRITE) == true) {
		for (int i = 0; i < packet->data_length; i++) {			
			if (i2c_write_byte(packet->data[i]) == false) {
				return STATUS_ERR_OVERFLOW;
			}
		}
	}
	else {
		return STATUS_ERR_OVERFLOW;
	}
	return STATUS_OK;
}

enum status_code i2c_master_write_packet_wait(struct i2c_master_packet *const packet) {	
	i2c_start();
	if (i2c_write_byte((packet->address << 1) | I2C_TRANSFER_WRITE) == true) {
		for (int i = 0; i < packet->data_length; i++) {			
			if (i2c_write_byte(packet->data[i]) == false) {
				i2c_stop();
				return STATUS_ERR_OVERFLOW;
			}
		}
	}
	else {
		i2c_stop();
		return STATUS_ERR_OVERFLOW;
	}
	i2c_stop();
	return STATUS_OK;
}
