/**
 * \file lcd.c
 *
 * \brief Explorer 16 LCD driver source file
 *
 * Copyright (c) 2016 Measurement Specialties. All rights reserved.
 *
 */

#include "hal.h"
#include <stdarg.h>
#include <stdio.h>

/*
 * private defines
 *
 */

#define LCD_delay       5                             // ~5mS
#define LCD_Startup     15                            // ~15mS

// Command set for Hitachi 44780U LCD display controller
#define LCD_CLEAR           0x01
#define LCD_HOME            0x02
#define LCD_CURSOR_BACK     0x10
#define LCD_CURSOR_FWD      0x14
#define LCD_PAN_LEFT        0x18
#define LCD_PAN_RIGHT       0x1C
#define LCD_CURSOR_OFF      0x0C
#define LCD_CURSOR_ON       0x0E
#define LCD_CURSOR_BLINK    0x0F
#define LCD_CURSOR_LINE2    0xC0

// Macros recommended for new labs
#define mLCD_PAN_RIGHT      lcd_put_cmd(LCD_PAN_RIGHT)
#define mLCD_PAN_LEFT       lcd_put_cmd(LCD_PAN_LEFT)
#define mLCD_CURSOR_BLINK   lcd_put_cmd(LCD_CURSOR_BLINK)
#define mLCD_HOME           lcd_put_cmd(LCD_HOME)
#define mLCD_CLEAR          lcd_put_cmd(LCD_CLEAR)

#define mCURSOR_ON          lcd_put_cmd(LCD_CURSOR_ON)
#define mCURSOR_OFF         lcd_put_cmd(LCD_CURSOR_OFF)
#define mCURSOR_FWD         lcd_put_cmd(LCD_CURSOR_FWD)
#define mCURSOR_BACK        lcd_put_cmd(LCD_CURSOR_BACK)
#define mCURSOR_LINE2       lcd_put_cmd(LCD_CURSOR_LINE2)

// display controller setup commands from page 46 of Hitachi datasheet
#define FUNCTION_SET        0x3C                        // 8 bit interface, 2 lines, 5x8 font
#define ENTRY_MODE          0x06                        // increment mode
#define DISPLAY_SETUP       0x0C                        // display on, cursor off, blink off

// Macro definitions (not recommended for new code)
#define LCDCurFwd()         lcd_put_cmd(LCD_CURSOR_FWD)   // legacy support
#define LCDCurBack()        lcd_put_cmd(LCD_CURSOR_BACK)  // legacy support
#define LCDPanLeft()        lcd_put_cmd(LCD_PAN_LEFT)     // legacy support
#define LCDPanRight()       lcd_put_cmd(LCD_PAN_RIGHT)    // legacy support
#define LCDClear()          lcd_put_cmd(LCD_CLEAR)        // legacy support
#define LCDPut              lcd_put_char                  // legacy support
#define LCDL1Home()         lcd_put_cur(0,0)              // legacy support
#define LCDL2Home()         lcd_put_cur(1,0)              // legacy support

/*
 * private variables
 *
 */
static char buffer[40];

/*
 * private functions
 *
 */
void lcd_put_char(unsigned char ch)
{
	PMADDR = 0x0001;
	PMDIN1 = ch;
	delay_ms(LCD_delay);
}

void lcd_put_cmd(unsigned char cmd)
{
	PMADDR = 0x0000;
	PMDIN1 = cmd;
	delay_ms(LCD_delay);
}

void lcd_put_cur(char row, char col)
{
	row <<= 6;
	lcd_put_cmd(0x80|row|col);
}

void lcd_put_str(char *ptr)
{
	while(*ptr) lcd_put_char(*(ptr++));
}

/*
 * public functions
 *
 */
void lcd_initialize(void)
{
	PMCON = 0x83BF;                 // configure the PMP
	PMMODE = 0x3FF;
	PMAEN = 0x0001;                 // display controller defines 4 character to setup LCD

	delay_ms(LCD_Startup);          // required by display controller to allow power to stabilize
	lcd_put_cmd(0x32);                // required by display initialization
	lcd_put_cmd(FUNCTION_SET);        // set interface size, # of lines and font
	lcd_put_cmd(DISPLAY_SETUP);       // turn on display and sets up cursor
	mLCD_CLEAR;
	lcd_put_cmd(ENTRY_MODE);          // set cursor movement direction
	lcd_put_cmd(LCD_CLEAR);
}

void lcd_set_position(uint8_t x, uint8_t y)
{
	lcd_put_cur(y, x);
}

void lcd_printf(char *format, ...)
{
	va_list args;
	va_start (args, format);
	vsprintf (buffer,format, args);
	lcd_put_str(buffer);
	va_end (args);
}

void lcd_clear(void)
{
	lcd_put_cmd(LCD_CLEAR);
}