/**
 * \file main.c
 *
 * \brief MEAS PICtail main source file
 *
 * Copyright (c) 2016 Measurement Specialties. All rights reserved.
 *
 */

#include "hal.h"
#include "lcd.h"

#include "htu21d.h"
#include "ms5637.h"
#include "tsys01.h"
#include "ms8607.h"

/*
 * configuration fuses
 *
 */
_CONFIG1(JTAGEN_OFF & FWDTEN_OFF)
_CONFIG2(FNOSC_PRI & POSCMOD_HS)

/*
 * private defines
 *
 */
#define SENSOR_COUNT	4

/*
 * private variables
 *
 */
enum display_state {
	display_state_initialize,
	display_state_main_menu,
	display_state_htu21d,
	display_state_ms5637,
	display_state_tsys01,
	display_state_ms8607,
};

static char* sensor_list[SENSOR_COUNT] = {
	"HTU21D",
	"MS5637",
	"TSYS01",
	"MS8607"
};

static int sensor_index = 0;
static int menu_offset = 0;
static enum display_state state = display_state_initialize;
static bool should_update_sensor = true;
static bool sensor_initialized = false;

/*
 * private functions
 *
 */
static void initialize_measure_timer(void) 
{
	T1CON = 0x00;                   //Stops the Timer1 and reset control reg.
    T1CONbits.TCS = T1CS_FOSC;      //Initialize Timer clock source
    T1CONbits.TCKPS = T1CKPS_256;    //Initialize the Timer clock prescaler
    TMR1 = 0x00;                    //Clear contents of the timer register
    PR1 = T1_1sec;                  //Load the Period register
    IPC0bits.T1IP = 0x01;           //Setup Timer1 interrupt for desired priority level
                                    // (This example assigns level 1 priority)
    IFS0bits.T1IF = 0;              //Clear the Timer1 interrupt status flag
    IEC0bits.T1IE = 1;              //Enable Timer1 interrupts
    T1CONbits.TON = T1ON;           //Start Timer1 with prescaler settings at 1:1 and
    //clock source set to the internal instruction cycle
}

static void initialize(void)
 {
	AD1PCFG = 0x001B;           // Configure PORTB<4,3,1,0> as I/O not analog
	
	TRIS_SW3 = INPUT;
	TRIS_SW4 = INPUT;
	TRIS_SW6 = INPUT;
	
	disable_all();
	
	lcd_initialize();
	initialize_measure_timer();
}

static void display_menu(void)
{
	lcd_clear();
	
	for (int i = 0; i < LCD_HEIGHT; i++) {
		lcd_printfxy(2, i, sensor_list[i + menu_offset]);
	}
	
	lcd_printfxy(0,				sensor_index - menu_offset, ">");
	
	lcd_printfxy(LCD_WIDTH - 3, 0, ">S3");
	lcd_printfxy(LCD_WIDTH - 3, 1, "vS4");
}

static void display_back_if_needed(void)
{
	if (is_sw4_pressed() || is_sw3_pressed()) {
		menu_offset = 0;
		sensor_index = 0;
		state = display_state_main_menu;
		disable_all();
		delay_ms(500);
		display_menu();
	}
}

/*
 * program entry point
 *
 */
int main()
{
	float humidity, pressure ,temperature;
	
	initialize();
	
	while(1) {
		switch (state) {
			case display_state_initialize:
                disable_all();
				lcd_printfxy(0,0, "TE Connectivity");
				lcd_printfxy(0,1, "MEAS PICtail");
				delay_ms(2000);
				display_menu();
				
				state = display_state_main_menu;
				break;
				
			case display_state_main_menu:
                disable_all();
				if (is_sw4_pressed()) {
					if (++sensor_index >= LCD_HEIGHT) {
						menu_offset++;
					}
					if (sensor_index >= SENSOR_COUNT) {
						sensor_index = 0;
					}
					if (menu_offset > SENSOR_COUNT - LCD_HEIGHT) {
						menu_offset = 0;
						sensor_index = 0;
					}
					display_menu();
				}
				if (is_sw3_pressed()) {
					sensor_initialized = false;
					should_update_sensor = true;
					lcd_clear();
					state = sensor_index + display_state_htu21d;		
				}
				break;
				
			case display_state_htu21d:
				if (sensor_initialized == false) {
					sensor_initialized = true;					
					enable_htu21d();	
					htu21_init();
				}
				if (should_update_sensor) {
					should_update_sensor = false;
					
					lcd_printfxy(0,0, "%s ", sensor_list[sensor_index]);
					if (htu21_read_temperature_and_relative_humidity(&temperature, &humidity) == htu21_status_ok) {
						lcd_printfxy(10,0, "%.1f%cC ", temperature, DEGREE_CHAR);
						lcd_printfxy(10,1, "%.0f%% ", humidity);
					} 
					else {
						lcd_printfxy(10,0, "----%cC ", DEGREE_CHAR);
						lcd_printfxy(10,1, "----%% ");
					}
				}
				display_back_if_needed();
				break;
				
			case display_state_ms5637:
				if (sensor_initialized == false) {
					sensor_initialized = true;					
					enable_ms5637();
					ms5637_init();
				}
				if (should_update_sensor) {
					should_update_sensor = false;
					
					lcd_printfxy(0,0, "%s ", sensor_list[sensor_index]);
					if (ms5637_read_temperature_and_pressure(&temperature, &pressure) == ms5637_status_ok) {
						lcd_printfxy(10,0, "%.1f%cC ", temperature, DEGREE_CHAR);
						lcd_printfxy(7,1, "%.1fhPa ", pressure);
					}
					else  {
						lcd_printfxy(10,0, "----%cC ", DEGREE_CHAR);
						lcd_printfxy(7,1,  "------hPa ");
					}
				}
				display_back_if_needed();
				break;
				
			case display_state_tsys01:
				if (sensor_initialized == false) {
					sensor_initialized = true;					
					enable_tsys01();
					tsys01_init();
				}
				if (should_update_sensor) {
					should_update_sensor = false;
					
					lcd_printfxy(0,0, "%s ", sensor_list[sensor_index]);
					if (tsys01_read_temperature(&temperature) == tsys01_status_ok) {
						lcd_printfxy(10,0, "%.1f%cC ", temperature, DEGREE_CHAR);
					}
					else {
						lcd_printfxy(10,0, "----%cC ", DEGREE_CHAR);
					}					
				}
				display_back_if_needed();
				break;
				
			case display_state_ms8607:
				if (sensor_initialized == false) {
					sensor_initialized = true;					
					enable_ms8607();
					ms8607_init();
				}
				if (should_update_sensor) {
					should_update_sensor = false;
					
					lcd_printfxy(0,0, "%s ", sensor_list[sensor_index]);
					if (ms8607_read_temperature_pressure_humidity(&temperature, &pressure, &humidity) == ms8607_status_ok) {
						lcd_printfxy(10,0, "%.1f%cC ", temperature, DEGREE_CHAR);
						lcd_printfxy(7,1,  "%.1fhPa ", pressure);
						lcd_printfxy(0,1,  "%.0f%% ", humidity);
					}
					else  {
						lcd_printfxy(10,0, "----%cC ", DEGREE_CHAR);
						lcd_printfxy(7,1,  "------hPa ");
						lcd_printfxy(0,1,  "----%% ");
					}
				}
				display_back_if_needed();
				break;
				
			
		}
	}

	return 0;
}

/*
 * interrupts
 *
 */
void __attribute__((interrupt, shadow, no_auto_psv)) _T1Interrupt(void)
{
    should_update_sensor = true;
    PR1 = T1_500ms;
    _T1IF = 0;
}








